#!/usr/bin/env python3

import ogr,os,sys,psycopg2
from PIL import Image,ImageDraw

def drawPolygon(geom, draw, w, h, minlng, maxlng, minlat, maxlat):
    for j in range(0, geom.GetGeometryCount()):
        item = geom.GetGeometryRef(j)
        for i in range(0, item.GetPointCount()-2):
            p1 = item.GetPoint(i)
            p1=(w*(p1[0]-minlng)/(maxlng-minlng),h-h*(p1[1]-minlat)/(maxlat-minlat))
            p2 = item.GetPoint(i + 1)
            p2=(w*(p2[0]-minlng)/(maxlng-minlng),h-h*(p2[1]-minlat)/(maxlat-minlat))
            draw.line(p1 + p2, fill=128)
        p1 = item.GetPoint(item.GetPointCount()-1)
        p1=(w*(p1[0]-minlng)/(maxlng-minlng),h-h*(p1[1]-minlat)/(maxlat-minlat))
        p2 = item.GetPoint(0)
        p2=(w*(p2[0]-minlng)/(maxlng-minlng),h-h*(p2[1]-minlat)/(maxlat-minlat))
        draw.line(p1 + p2, fill=128)
    return
#server information
hostip="localhost"
myport=5433
myuser='postgres'
mypwd='postgres'
db='mydb'

os.chdir('/tmp/CHN_adm')

conn=psycopg2.connect(host=hostip,port=myport,user=myuser,password=mypwd,database=db)
cur=conn.cursor()

cur.execute('DROP TABLE IF EXISTS public.chn_adm1')
dirver=ogr.GetDriverByName('ESRT Shapefile')
cur.execute('CREATE TABLE public.CHN_adm1 (gid int4 Not NULL primary key, name0 varchar(50), name1 varchar(50), name2 varchar(50), name3 varchar(50), geom public.geometry) ;')
shapefile=dirver.Open('CHN_adm1.shp')
if shapefile is None:
    print('Could not open file')
    sys.exit(1)
layer0=shapefile.GetLayer()
for i in range(layer0.GetFeatureCount()):
    feature=layer0.GetFeature(i)
    name0 = feature.GetField("NAME_0")#.decode("utf-8")
    name1 = feature.GetField("NAME_1")#.decode("utf-8")
    name2 = feature.GetField("NAME_2")#.decode("utf-8")
    name3 = feature.GetField("NAME_3")#.decode("utf-8")
    gid=i
    wkt=feature.GetGeometryRef().ExportToWkt()
    cur.execute("INSERT INTO public.CHN_adm1  (gid, name0, name1, name2, name3, geom) " +"VALUES (%s, %s, %s, %s, %s, ST_GeometryFromText(%s, " +"4326));", (gid, name0, name1, name2, name3, wkt))
conn.commit()
print('Shapefile import successfully!')

#export shapefile
if os.path.exists('Output.shp'):
    dirver.DeleteDataSource('Output.shp')
outDataSource=dirver.DeleteDataSource('Output.shp')
if outDataSource is None:
    print('Create Output.shp failed')
    sys.exit(1)
OutLayer=outDataSource.CreateLayer('outlayer',geom_type=ogr.wkbMultiPolygon)
gid=ogr.FieldDefn("gid", ogr.OFTInteger)
OutLayer.CreateField(gid)
name0 = ogr.FieldDefn( "name0", ogr.OFTString )
name0.SetWidth( 30 )
OutLayer.CreateField(name0)
name1 = ogr.FieldDefn( "name1", ogr.OFTString )
name1.SetWidth( 30 )
OutLayer.CreateField(name1)
name2 = ogr.FieldDefn( "name2", ogr.OFTString )
name2.SetWidth( 30 )
OutLayer.CreateField(name2)
name3 = ogr.FieldDefn( "name3", ogr.OFTString )
name3.SetWidth( 30 )
OutLayer.CreateField(name3)
outDefn = OutLayer.GetLayerDefn()

cur.execute('select gid, name0, name1, name2, name3, ST_AsText(geom) AS geom from public.CHN_adm1;')
shp=cur.fetchall()
for row in shp:
  # create a new feature
    outFeature = ogr.Feature(outDefn)
    # set the geometry and attributes
    fid=row[0]
    name0=row[1]
    name1=row[2]
    name2=row[3]
    name3=row[4]
    geo = ogr.CreateGeometryFromWkt(row[5])
    outFeature.SetGeometry(geo)
    outFeature.SetField('gid', fid)
    outFeature.SetField('name0',name0)
    outFeature.SetField('name1',name1)
    outFeature.SetField('name2',name2)
    outFeature.SetField('name3',name3)
    # save the feature to the output
    OutLayer.CreateFeature(outFeature)
    outFeature.Destroy()
print('export shapefile successfully!')


cur.execute("SELECT ST_XMin(bbox.geom), ST_XMax(bbox.geom), ST_YMin(bbox.geom), ST_YMax(bbox.geom) FROM (SELECT ST_Envelope(ST_Union(geom)) AS geom FROM public.CHN_adm1) AS bbox")
bbox = cur.fetchone()
minlng = bbox[0]
maxlng = bbox[1]
minlat = bbox[2]
maxlat = bbox[3]
w = 1280
h = int(w * (maxlat - minlat) / (maxlng - minlng))
img = Image.new("RGB", (w, h), "white")
draw = ImageDraw.Draw(img)
#draw geometry
cur.execute('select gid, name0, name1, name2, name3, ST_AsText(geom) AS geom from public.CHN_adm1;')
shp=cur.fetchall()
for row in shp:
    # print row[1]
    ob = ogr.CreateGeometryFromWkt(row[5])
    gtype=ob.GetGeometryName()
    #print(gtype)
    if gtype=='MULTIPOLYGON':
        for j in range(0, ob.GetGeometryCount()):
                item = ob.GetGeometryRef(j)
                drawPolygon(item, draw,w, h, minlng, maxlng, minlat, maxlat)
    else:
        drawPolygon(ob, draw,w, h, minlng, maxlng, minlat, maxlat)
#save image
img.save('China.jpg', 'JPEG')
print('Draw shp successfully!')
cur.close()
conn.close()


#===============================================================================
# # get the attributes in this layer
# layerDefn = layer.GetLayerDefn()
# for i in range(layerDefn.GetFieldCount()):
#     fieldName=layerDefn.GetFieldDefn(i).GetNameRef()
#     fieldType=layerDefn.GetFieldDefn(i).GetType()
#===============================================================================



#if __name__=='__main__':