# python 3.4.3
# -*- coding: utf-8 -*-

"""
Created on 03 11 2016
@author: wangyuxia
"""

import ogr
import sys
import psycopg2
import os.path
from PIL import Image,ImageDraw
sys.path.append("/home/wangyuxia/PycharmProjects/PyTest")

def drawPolygon(geom, draw, w, h, minlng, maxlng, minlat, maxlat):
    for j in range(0, geom.GetGeometryCount()):
        item = geom.GetGeometryRef(j)
        for i in range(0, item.GetPointCount()-2):
            p1 = item.GetPoint(i)
            p1=(w*(p1[0]-minlng)/(maxlng-minlng),h-h*(p1[1]-minlat)/(maxlat-minlat))
            p2 = item.GetPoint(i + 1)
            p2=(w*(p2[0]-minlng)/(maxlng-minlng),h-h*(p2[1]-minlat)/(maxlat-minlat))
            draw.line(p1 + p2, fill=128)
        p1 = item.GetPoint(item.GetPointCount()-1)
        p1=(w*(p1[0]-minlng)/(maxlng-minlng),h-h*(p1[1]-minlat)/(maxlat-minlat))
        p2 = item.GetPoint(0)
        p2=(w*(p2[0]-minlng)/(maxlng-minlng),h-h*(p2[1]-minlat)/(maxlat-minlat))
        draw.line(p1 + p2, fill=128)
    return

# import the driver for the shapefile data
driver = ogr.GetDriverByName('ESRI Shapefile')
filename = "/home/wangyuxia/Documents/CHN_adm/CHN_adm2.shp"
dataSource = driver.Open(filename,0)
if dataSource is None:
    print("could not open the data")
    sys.exit(1)

#read the data layer
layer = dataSource.GetLayer(0)
n = layer.GetFeatureCount()
print("the feature count is : ",n)

# connect to postgresql
conn = psycopg2.connect(database="postgres", user="postgres", password="123456", host="127.0.0.1", port="5432")
print("Opened database successfully")
cur = conn.cursor()
cur.execute("DROP TABLE IF EXISTS china")
cur.execute("create table if not exists china(gid serial primary key, name_1 character varying(75),\
             name_2 character varying(75),geom geometry)")
layer = dataSource.GetLayer(0)
for i in range(layer.GetFeatureCount()):
    feature = layer.GetFeature(i)
    #gid = feature.GetField("gid").decode("latin-1")
    gid = i
    name_1 = feature.GetField("name_1")
    name_2 = feature.GetField("name_2")

    wkt = feature.GetGeometryRef().ExportToWkt()
    cur.execute("INSERT INTO china (gid,name_1,name_2,geom) " +"VALUES (%s, %s, %s, ST_GeometryFromText(%s, " +"4326))", (gid,name_1.encode("utf8"),name_2.encode("utf8"), wkt))

conn.commit()
print ("shapefile imported into postgresql successfully")


if os.path.exists("chinaOut.shp"):
    driver.DeleteDataSource("chinaOut.shp")
outDs = driver.CreateDataSource("chinaOut.shp")
if outDs is None:
    print ("could not create chinaOut.shp")
    sys.exit(1)
outLayer = outDs.CreateLayer("output",geom_type = ogr.wkbMultiPolygon)
gid = ogr.FieldDefn("gid", ogr.OFTInteger)
outLayer.CreateField(gid)
name_1 = ogr.FieldDefn("name_1",ogr.OFTString)
name_1.SetWidth(30)
outLayer.CreateField(name_1)
name_2 = ogr.FieldDefn("name_2",ogr.OFTString)
name_2.SetWidth(30)
outLayer.CreateField(name_2)

featureDefn = layer.GetLayerDefn()

cur.execute("select gid, name_1, name_2, ST_AsText(geom) AS geom from china ")
shp=cur.fetchall()
for row in shp:
    outFeature = ogr.Feature(featureDefn)
    gid=row[0]
    name_1=row[1]
    name_2=row[2]
    geo = ogr.CreateGeometryFromWkt(row[3])
    outFeature.SetGeometry(geo)
    outFeature.SetField("gid", gid)
    outFeature.SetField("name_1",name_1)
    outFeature.SetField("name_2",name_2)

    outLayer.CreateFeature(outFeature)
    outFeature.Destroy()

print("shapefile exported successfully!")

cur.execute("SELECT ST_XMin(bbox.geom),ST_XMax(bbox.geom),ST_YMin(bbox.geom),\
ST_YMax(bbox.geom) FROM (SELECT ST_Envelope(ST_Union(geom)) AS geom FROM china) AS bbox")
bbox = cur.fetchone()
minlng = bbox[0]
maxlng = bbox[1]
minlat = bbox[2]
maxlat = bbox[3]
w = 1280
h = int(w * (maxlat - minlat) / (maxlng - minlng))
img = Image.new("RGB", (w, h), "white")
draw = ImageDraw.Draw(img)
#draw geometry
cur.execute("select gid, name_1, name_2, ST_AsText(geom) AS geom from china ")
shp=cur.fetchall()
for row in shp:
    ob = ogr.CreateGeometryFromWkt(row[3])
    gtype=ob.GetGeometryName()
    if gtype=='MULTIPOLYGON':
        for j in range(0, ob.GetGeometryCount()):
                item = ob.GetGeometryRef(j)
                drawPolygon(item, draw,w, h, minlng, maxlng, minlat, maxlat)
    else:
        drawPolygon(ob, draw,w, h, minlng, maxlng, minlat, maxlat)
#save image
img.save('China.jpg', 'JPEG')
print('Draw shp successfully!')
cur.close()
conn.close()


